@@ -247,7 +247,7 @@ namespace ojph {
     assert(fh == 0);
     fh = fopen(filename, "rb");
     if (fh == 0)
-      OJPH_ERROR(0x030000001, "Unable to open file %s", filename);
+      OJPH_ERROR(0x03000001, "Unable to open file %s", filename);
     fname = filename;
 
     // read magic number
@@ -255,27 +255,27 @@ namespace ojph {
     if (fread(t, 1, 2, fh) != 2)
     {
       close();
-      OJPH_ERROR(0x030000002, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000002, "Error reading file %s", filename);
     }
 
     // check magic number
     if (t[0] != 'P' || (t[1] != '5' && t[1] != '6'))
     {
       close();
-      OJPH_ERROR(0x030000003, "unknown file type for file %s", filename);
+      OJPH_ERROR(0x03000003, "unknown file type for file %s", filename);
     }
 
     size_t len = strlen(filename);
     if (t[1] == '5' && strncmp(filename + len - 4, ".pgm", 4) != 0)
     {
       close();
-      OJPH_ERROR(0x030000004, "wrong file extension, a file with "
+      OJPH_ERROR(0x03000004, "wrong file extension, a file with "
         "keyword P5 must have a .pgm extension for file %s", filename);
     }
     if (t[1] == '6' && strncmp(filename + len - 4, ".ppm", 4) != 0)
     {
       close();
-      OJPH_ERROR(0x030000005, "wrong file extension, a file with keyword P6 "
+      OJPH_ERROR(0x03000005, "wrong file extension, a file with keyword P6 "
         "must have a .ppm extension for file %s", filename);
     }
 
@@ -287,7 +287,7 @@ namespace ojph {
     if (fscanf(fh, "%d %d %d", &width, &height, &max_val) != 3)
     {
       close();
-      OJPH_ERROR(0x030000006, "error in file format for file %s", filename);
+      OJPH_ERROR(0x03000006, "error in file format for file %s", filename);
     }
     num_ele_per_line = num_comps * width;
     bytes_per_sample = max_val > 255 ? 2 : 1;
@@ -309,7 +309,7 @@ namespace ojph {
           temp_buf = malloc(temp_buf_byte_size);
         if (temp_buf == NULL) { // failed to allocate memory
           if (t) free(t); // the original buffer is still valid
-          OJPH_ERROR(0x030000007, "error allocating memory");
+          OJPH_ERROR(0x03000007, "error allocating memory");
         }
       }
       else
@@ -347,7 +347,7 @@ namespace ojph {
       if (result != num_ele_per_line)
       {
         close();
-        OJPH_ERROR(0x030000011, "not enough data in file %s", fname);
+        OJPH_ERROR(0x03000011, "not enough data in file %s", fname);
       }
       if (++cur_line >= height)
       {
@@ -394,17 +394,17 @@ namespace ojph {
         if (strncmp(".ppm", filename + len - 4, 4) == 0)
         {
           filename[len - 2] = 'g'; 
-          OJPH_WARN(0x03000001, "file was renamed %s\n", filename);
+          OJPH_WARN(0x03000021, "file was renamed %s\n", filename);
         }
         if (strncmp(".PPM", filename + len - 4, 4) == 0)
         {
           filename[len - 2] = 'G';
-          OJPH_WARN(0x03000002, "file was renamed %s\n", filename);
+          OJPH_WARN(0x03000022, "file was renamed %s\n", filename);
         }
       }
       fh = fopen(filename, "wb");
       if (fh == NULL)
-        OJPH_ERROR(0x030000021,
+        OJPH_ERROR(0x03000023,
           "unable to open file %s for writing", filename);
 
       fprintf(fh, "P5\n%d %d\n%d\n", width, height, (1 << bit_depth) - 1);
@@ -419,22 +419,22 @@ namespace ojph {
         if (strncmp(".pgm", filename + len - 4, 4) == 0)
         {
           filename[len - 2] = 'p';
-          OJPH_WARN(0x03000003, "file was renamed %s\n", filename);
+          OJPH_WARN(0x03000024, "file was renamed %s\n", filename);
         }
         if (strncmp(".PGM", filename + len - 4, 4) == 0)
         {
           filename[len - 2] = 'P';
-          OJPH_WARN(0x03000004, "file was renamed %s\n", filename);
+          OJPH_WARN(0x03000025, "file was renamed %s\n", filename);
         }
       }
       fh = fopen(filename, "wb");
       if (fh == NULL)
-        OJPH_ERROR(0x030000022,
+        OJPH_ERROR(0x03000026,
           "unable to open file %s for writing", filename);
       int result = //the number of written characters
         fprintf(fh, "P6\n%d %d\n%d\n", width, height, (1 << bit_depth) - 1);
       if (result == 0)
-        OJPH_ERROR(0x030000023, "error writing to file %s", filename);
+        OJPH_ERROR(0x03000027, "error writing to file %s", filename);
       buffer_size = width * num_components * bytes_per_sample;
       buffer = (ui8*)malloc(buffer_size);
     }
@@ -448,7 +448,7 @@ namespace ojph {
   {
     assert(fh == NULL); //configure before opening
     if (num_components != 1 && num_components != 3)
-      OJPH_ERROR(0x030000031,
+      OJPH_ERROR(0x03000031,
         "ppm supports 3 colour components, while pgm supports 1");
     this->width = width;
     this->height = height;
@@ -530,12 +530,257 @@ namespace ojph {
       size_t result = fwrite(buffer,
                               bytes_per_sample, samples_per_line, fh);
       if (result != samples_per_line)
-        OJPH_ERROR(0x030000042, "error writing to file %s", fname);
+        OJPH_ERROR(0x03000041, "error writing to file %s", fname);
     }
     return 0;
   }
 
   ////////////////////////////////////////////////////////////////////////////
+  //
+  //
+  //
+  //
+  //
+  ////////////////////////////////////////////////////////////////////////////
+
+  /////////////////////////////////////////////////////////////////////////////
+  void pfm_in::open(const char *filename)
+  {
+    assert(fh == 0);
+    fh = fopen(filename, "rb");
+    if (fh == 0)
+      OJPH_ERROR(0x03000051, "Unable to open file %s", filename);
+    fname = filename;
+
+    // read magic number
+    char t[2];
+    if (fread(t, 1, 2, fh) != 2)
+    {
+      close();
+      OJPH_ERROR(0x03000052, "Error reading file %s", filename);
+    }
+
+    // check magic number
+    if (t[0] != 'P' || (t[1] != 'F' && t[1] != 'f'))
+    {
+      close();
+      OJPH_ERROR(0x03000053, "Unknown file type for file %s", filename);
+    }
+
+    // set number of components based on file-type
+    num_comps = t[1] == 'f' ? 1 : 3;
+    eat_white_spaces(fh);
+
+    // read width, height and max value in header
+    if (fscanf(fh, "%d %d", &width, &height) != 2)
+    {
+      close();
+      OJPH_ERROR(0x03000054, 
+        "Error reading width and height in file %s", filename);
+    }
+    eat_white_spaces(fh);
+
+    // little or big-endian
+    if (fscanf(fh, "%f", &scale) != 1)
+    {
+      close();
+      OJPH_ERROR(0x03000055, "Error reading scale in file %s", filename);
+    }
+    little_endian = scale < 0.0f;
+    scale = std::abs(scale);
+
+    fgetc(fh);
+    start_of_data = ojph_ftell(fh);
+
+    // alloc. linebuffer to hold a line of image data, if more than 1 comp.
+    if (temp_buf_byte_size < num_comps * (size_t)width * sizeof(float))
+    {
+      if (alloc_p == NULL)
+      {
+        temp_buf_byte_size = num_comps * (size_t)width * sizeof(float);
+        void* t = temp_buf;
+        if (temp_buf)
+          temp_buf = (float*)realloc(temp_buf, temp_buf_byte_size);
+        else
+          temp_buf = (float*)malloc(temp_buf_byte_size);
+        if (temp_buf == NULL) { // failed to allocate memory
+          if (t) free(t); // the original buffer is still valid
+          OJPH_ERROR(0x03000056, "Error allocating memory");
+        }
+      }
+      else
+      {
+        assert(temp_buf_byte_size == 0); //cannot reallocate the buffer
+        temp_buf_byte_size = num_comps * (size_t)width * sizeof(float);
+        alloc_p->pre_alloc_data<float>(temp_buf_byte_size, 0);
+      }
+    }
+    cur_line = 0;
+  }
+
+  /////////////////////////////////////////////////////////////////////////////
+  void pfm_in::finalize_alloc()
+  {
+    if (alloc_p == NULL)
+      return;
+    temp_buf = alloc_p->post_alloc_data<float>(num_comps * (size_t)width, 0);
+  }
+
+  /////////////////////////////////////////////////////////////////////////////
+  ui32 pfm_in::read(const line_buf* line, ui32 comp_num)
+  {
+    assert(temp_buf_byte_size != 0 );
+    assert(fh != 0 && comp_num < num_comps);
+    assert(line->size >= width);
+
+    if (comp_num == 0)
+    {
+      si64 loc = start_of_data;
+      loc += (size_t)(height-1 - cur_line) * (size_t)num_comps 
+           * (size_t)width * sizeof(float);
+      if (ojph_fseek(fh, loc, SEEK_SET) != 0)
+      {
+        close();
+        OJPH_ERROR(0x03000061, "Error seeking in file %s", fname);
+      }
+      size_t result = 
+        fread(temp_buf, sizeof(float), (size_t)num_comps * (size_t)width, fh);
+      if (result != (size_t)num_comps * (size_t)width)
+      {
+        close();
+        OJPH_ERROR(0x03000062, "Not enough data in file %s", fname);
+      }
+      if (++cur_line >= height)
+        cur_line = 0;
+    }
+
+    union {
+      si32* s;
+      ui32* u;
+      float* f;
+    } sp, dp;
+
+    if (little_endian)
+    {
+      ui32 shift = 32 - bit_depth[comp_num];
+      sp.f = temp_buf + comp_num;
+      dp.f = line->f32;
+      if (shift)
+        for (ui32 i = width; i > 0; --i, sp.f += num_comps) 
+        {
+          si32 s = *sp.s;
+          s >>= shift;
+          *dp.s++ = s;
+        }
+      else
+        for (ui32 i = width; i > 0; --i, sp.f += num_comps)
+          *dp.f++ = *sp.f;
+    }
+    else {
+      ui32 shift = 32 - bit_depth[comp_num];
+      sp.f = temp_buf + comp_num;
+      dp.f = line->f32;
+      if (shift)
+        for (ui32 i = width; i > 0; --i, sp.f += num_comps) {
+          ui32 u = be2le(*sp.u);
+          si32 s = *(si32*)&u;
+          s >>= shift;
+          *dp.s++ = s;
+        }
+      else
+        for (ui32 i = width; i > 0; --i, sp.f += num_comps)
+          *dp.u++ = be2le(*sp.u);
+    }
+
+    return width;
+  }
+
+  ////////////////////////////////////////////////////////////////////////////
+  //
+  //
+  //
+  //
+  //
+  ////////////////////////////////////////////////////////////////////////////
+
+  ////////////////////////////////////////////////////////////////////////////
+  void pfm_out::open(char* filename)
+  {
+    assert(fh == NULL && buffer == NULL);
+    fh = fopen(filename, "wb");
+    if (fh == NULL)
+      OJPH_ERROR(0x03000071,
+        "Unable to open file %s for writing", filename);
+    int result = //the number of written characters
+      fprintf(fh, "P%c\n%d %d\n%f\n", 
+        num_components > 1 ? 'F' : 'f', width, height, scale);
+    if (result == 0)
+      OJPH_ERROR(0x03000072, "error writing to file %s", filename);
+    buffer_size = (size_t)width * num_components * sizeof(float);
+    buffer = (float*)malloc(buffer_size);
+    fname = filename;
+    cur_line = 0;
+    start_of_data = ojph_ftell(fh);
+  }
+
+  ////////////////////////////////////////////////////////////////////////////
+  void pfm_out::configure(ui32 width, ui32 height, ui32 num_components, 
+                          float scale, ui32* bit_depth)
+  {
+    assert(fh == NULL); //configure before opening
+    if (num_components != 1 && num_components != 3)
+      OJPH_ERROR(0x03000081,
+        "pfm supports 1 or 3 colour components, not %d", num_components);
+    this->width = width;
+    this->height = height;
+    this->num_components = num_components;
+    this->scale = scale < 0.0f ? scale : -scale;
+    for (ui32 c = 0; c < num_components; ++c)
+      this->bit_depth[c] = bit_depth[c];
+  }
+
+  ////////////////////////////////////////////////////////////////////////////
+  ui32 pfm_out::write(const line_buf* line, ui32 comp_num)
+  {
+    assert(fh);
+
+    ui32 shift = 32 - bit_depth[comp_num];
+    union {
+      ui32* u;
+      float* f;
+    } sp, dp;
+
+    dp.f = buffer + comp_num;
+    sp.f = line->f32;
+
+    if (shift)
+      for (ui32 i = width; i > 0; --i, dp.f += num_components, ++sp.f)
+      {
+        ui32 u = *sp.u;
+        u <<= shift;
+        *dp.u = u;
+      }
+    else
+      for (ui32 i = width; i > 0; --i, dp.f += num_components)
+        *dp.f = *sp.f++;
+
+    if (comp_num == num_components - 1)
+    {
+      size_t samples_per_line = num_components * (size_t)width;
+      si64 loc = start_of_data;
+      loc += (height - 1 - cur_line)* samples_per_line * sizeof(float);
+      if (ojph_fseek(fh, loc, SEEK_SET) != 0)
+        OJPH_ERROR(0x03000082, "Error seeking in file %s", fname);
+      size_t result = fwrite(buffer, sizeof(float), samples_per_line, fh);
+      if (result != samples_per_line)
+        OJPH_ERROR(0x03000083, "error writing to file %s", fname);
+      ++cur_line;
+    }
+
+    return 0;
+  }
+
+ ////////////////////////////////////////////////////////////////////////////
  //
  //
  //
@@ -548,7 +793,7 @@ namespace ojph {
   {
     tiff_handle = NULL;
     if ((tiff_handle = TIFFOpen(filename, "r")) == NULL)
-      OJPH_ERROR(0x0300000B1, "Unable to open file %s", filename);
+      OJPH_ERROR(0x03000091, "Unable to open file %s", filename);
     fname = filename;
 
     ui32 tiff_width = 0;
@@ -588,7 +833,7 @@ namespace ojph {
     // allocate linebuffer to hold a line of image data
     line_buffer = malloc(bytes_per_line);
     if (NULL == line_buffer)
-      OJPH_ERROR(0x0300000B2, "Unable to allocate %d bytes for line_buffer[] "
+      OJPH_ERROR(0x03000092, "Unable to allocate %d bytes for line_buffer[] "
         "for file %s", bytes_per_line, filename);
       
     cur_line = 0;
@@ -596,7 +841,7 @@ namespace ojph {
     // Error on known incompatilbe input formats
     if( tiff_bits_per_sample != 8 && tiff_bits_per_sample != 16 )
     {
-      OJPH_ERROR(0x0300000B3, "\nTIFF IO is currently limited"
+      OJPH_ERROR(0x03000093, "\nTIFF IO is currently limited"
         " to files with TIFFTAG_BITSPERSAMPLE=8 and TIFFTAG_BITSPERSAMPLE=16 \n"
         "input file = %s has TIFFTAG_BITSPERSAMPLE=%d", 
         filename, tiff_bits_per_sample);
@@ -604,14 +849,14 @@ namespace ojph {
 
     if( TIFFIsTiled( tiff_handle ) )
     {
-      OJPH_ERROR(0x0300000B4, "\nTIFF IO is currently limited to TIF files "
+      OJPH_ERROR(0x03000094, "\nTIFF IO is currently limited to TIF files "
         "without tiles. \nInput file %s has been detected as tiled", filename);
     }
 
     if(PHOTOMETRIC_RGB != tiff_photometric && 
        PHOTOMETRIC_MINISBLACK != tiff_photometric )
     {
-      OJPH_ERROR(0x0300000B5, "\nTIFF IO is currently limited to "
+      OJPH_ERROR(0x03000095, "\nTIFF IO is currently limited to "
         "TIFFTAG_PHOTOMETRIC=PHOTOMETRIC_MINISBLACK=%d and "
         "PHOTOMETRIC_RGB=%d. \nInput file %s has been detected "
         "TIFFTAG_PHOTOMETRIC=%d", 
@@ -620,7 +865,7 @@ namespace ojph {
 
     if( tiff_samples_per_pixel > 4 )
     {
-      OJPH_ERROR(0x0300000B6, "\nTIFF IO is currently limited to "
+      OJPH_ERROR(0x03000096, "\nTIFF IO is currently limited to "
         "TIFFTAG_SAMPLESPERPIXEL=4 \nInput file %s has been detected with "
         "TIFFTAG_SAMPLESPERPIXEL=%d",
         filename, tiff_samples_per_pixel);
@@ -642,7 +887,7 @@ namespace ojph {
       line_buffer_for_planar_support_uint8 = 
         (uint8_t*)calloc(width, sizeof(uint8_t));
       if (NULL == line_buffer_for_planar_support_uint8)
-        OJPH_ERROR(0x0300000B7, "Unable to allocate %d bytes for "
+        OJPH_ERROR(0x03000097, "Unable to allocate %d bytes for "
           "line_buffer_for_planar_support_uint8[] for file %s", 
           width * sizeof(uint8_t), filename);
     }
@@ -652,7 +897,7 @@ namespace ojph {
       line_buffer_for_planar_support_uint16 = 
         (uint16_t*)calloc(width, sizeof(uint16_t));
       if (NULL == line_buffer_for_planar_support_uint16)
-        OJPH_ERROR(0x0300000B8, "Unable to allocate %d bytes for "
+        OJPH_ERROR(0x03000098, "Unable to allocate %d bytes for "
           "line_buffer_for_planar_support_uint16[] for file %s", 
           width * sizeof(uint16_t), filename);
     }
@@ -664,7 +909,7 @@ namespace ojph {
   void tif_in::set_bit_depth(ui32 num_bit_depths, ui32* bit_depth)
   {
     if (num_bit_depths < 1)
-      OJPH_ERROR(0x030000B9, "one or more bit_depths must be provided");
+      OJPH_ERROR(0x030000A1, "one or more bit_depths must be provided");
     ui32 last_bd_idx = 0;
     for (ui32 i = 0; i < 4; ++i)
     {
@@ -673,7 +918,7 @@ namespace ojph {
 
       if (bd > 32 || bd < 1)
       {
-        OJPH_ERROR(0x0300000BA, 
+        OJPH_ERROR(0x030000A2, 
           "bit_depth = %d, this must be an integer from 1-32", bd);
       }
       this->bit_depth[i] = bd;
@@ -809,20 +1054,20 @@ namespace ojph {
     }
     if (max_bitdepth > 16)
     {
-      OJPH_WARN(0x0300000C2, "TIFF output is currently limited to files "
+      OJPH_WARN(0x030000B1, "TIFF output is currently limited to files "
         "with max_bitdepth = 16, the source codestream has max_bitdepth=%d"
         ", the decoded data will be truncated to 16 bits", max_bitdepth);
     }
     if (num_components > 4)
     {
-      OJPH_ERROR(0x0300000C3, "TIFF IO is currently limited to files with "
+      OJPH_ERROR(0x030000B2, "TIFF IO is currently limited to files with "
         "num_components=1 to 4");
     }
 
     assert(tiff_handle == NULL && buffer == NULL);
     if ((tiff_handle = TIFFOpen(filename, "w")) == NULL)
     {
-      OJPH_ERROR(0x0300000C1, "unable to open file %s for writing", filename);
+      OJPH_ERROR(0x030000B3, "unable to open file %s for writing", filename);
     }
 
     buffer_size = width * num_components * bytes_per_sample;
@@ -1014,7 +1259,7 @@ namespace ojph {
       {
         int result = TIFFWriteScanline(tiff_handle, buffer, cur_line++);
         if (result != 1)
-          OJPH_ERROR(0x0300000C4, "error writing to file %s", fname);
+          OJPH_ERROR(0x030000C1, "error writing to file %s", fname);
       }
     return 0;
   }
@@ -1034,7 +1279,7 @@ namespace ojph {
     assert(fh == NULL);
     fh = fopen(filename, "rb");
     if (fh == 0)
-      OJPH_ERROR(0x03000051, "Unable to open file %s", filename);
+      OJPH_ERROR(0x030000D1, "Unable to open file %s", filename);
 
     //need to extract info from filename
 
@@ -1062,7 +1307,7 @@ namespace ojph {
     if (result != width[comp_num])
     {
       close();
-      OJPH_ERROR(0x03000061, "not enough data in file %s", fname);
+      OJPH_ERROR(0x030000E1, "not enough data in file %s", fname);
     }
 
     if (bytes_per_sample[comp_num] == 1)
@@ -1088,11 +1333,11 @@ namespace ojph {
                              ui32 num_downsamplings, const point *subsampling)
   {
     if (num_components != 1 && num_components !=3)
-      OJPH_ERROR(0x03000071, "yuv_in support 1 or 3 components");
+      OJPH_ERROR(0x030000F1, "yuv_in support 1 or 3 components");
     this->num_com = num_components;
 
     if (num_downsamplings < 1)
-      OJPH_ERROR(0x03000072, "one or more downsampling must be provided");
+      OJPH_ERROR(0x030000F2, "one or more downsampling must be provided");
 
     ui32 last_downsamp_idx = 0;
     for (ui32 i = 0; i < num_components; ++i)
@@ -1114,7 +1359,7 @@ namespace ojph {
   void yuv_in::set_bit_depth(ui32 num_bit_depths, ui32* bit_depth)
   {
     if (num_bit_depths < 1)
-      OJPH_ERROR(0x03000081, "one or more bit_depths must be provided");
+      OJPH_ERROR(0x03000101, "one or more bit_depths must be provided");
     ui32 last_bd_idx = 0;
     for (ui32 i = 0; i < 3; ++i)
     {
@@ -1156,7 +1401,7 @@ namespace ojph {
     assert(fh == NULL); //configure before open
     fh = fopen(filename, "wb");
     if (fh == 0)
-      OJPH_ERROR(0x03000091, "Unable to open file %s", filename);
+      OJPH_ERROR(0x03000111, "Unable to open file %s", filename);
     fname = filename;
   }
 
@@ -1199,7 +1444,7 @@ namespace ojph {
         *dp++ = (ui16)val;
       }
       if (fwrite(buffer, 2, w, fh) != w)
-        OJPH_ERROR(0x030000A1, "unable to write to file %s", fname);
+        OJPH_ERROR(0x03000121, "unable to write to file %s", fname);
     }
     else
     {
@@ -1213,7 +1458,7 @@ namespace ojph {
         *dp++ = (ui8)val;
       }
       if (fwrite(buffer, 1, w, fh) != w)
-        OJPH_ERROR(0x030000A2, "unable to write to file %s", fname);
+        OJPH_ERROR(0x03000122, "unable to write to file %s", fname);
     }
 
     return w;
@@ -1233,7 +1478,7 @@ namespace ojph {
     assert(fh == NULL);
     fh = fopen(filename, "rb");
     if (fh == NULL)
-      OJPH_ERROR(0x030000C1, "Unable to open file %s", filename);
+      OJPH_ERROR(0x03000131, "Unable to open file %s", filename);
 
     cur_line = 0;
     bytes_per_sample = (bit_depth + 7) >> 3;
@@ -1251,7 +1496,7 @@ namespace ojph {
     if (result != width)
     {
       close();
-      OJPH_ERROR(0x030000C2, "not enough data in file %s", fname);
+      OJPH_ERROR(0x03000132, "not enough data in file %s", fname);
     }
 
     if (bytes_per_sample > 3)
@@ -1360,7 +1605,7 @@ namespace ojph {
     assert(fh == NULL); //configure before open
     fh = fopen(filename, "wb");
     if (fh == 0)
-      OJPH_ERROR(0x03000091, "Unable to open file %s", filename);
+      OJPH_ERROR(0x03000141, "Unable to open file %s", filename);
     fname = filename;
   }
 
@@ -1404,7 +1649,7 @@ namespace ojph {
         *dp++ = (ui32)val;
       }
       if (fwrite(buffer, bytes_per_sample, width, fh) != width)
-        OJPH_ERROR(0x030000B1, "unable to write to file %s", fname);
+        OJPH_ERROR(0x03000151, "unable to write to file %s", fname);
     }
     else if (bytes_per_sample > 2)
     {
@@ -1420,7 +1665,7 @@ namespace ojph {
         dp = (ui32*)((ui8*)dp + 3);
       }
       if (fwrite(buffer, bytes_per_sample, width, fh) != width)
-        OJPH_ERROR(0x030000B2, "unable to write to file %s", fname);
+        OJPH_ERROR(0x03000152, "unable to write to file %s", fname);
     }
     else if (bytes_per_sample > 1)
     {
@@ -1434,7 +1679,7 @@ namespace ojph {
         *dp++ = (ui16)val;
       }
       if (fwrite(buffer, bytes_per_sample, width, fh) != width)
-        OJPH_ERROR(0x030000B3, "unable to write to file %s", fname);
+        OJPH_ERROR(0x03000153, "unable to write to file %s", fname);
     }
     else
     {
@@ -1448,7 +1693,7 @@ namespace ojph {
         *dp++ = (ui8)val;
       }
       if (fwrite(buffer, bytes_per_sample, width, fh) != width)
-        OJPH_ERROR(0x030000B4, "unable to write to file %s", fname);
+        OJPH_ERROR(0x03000154, "unable to write to file %s", fname);
     }
 
     return width;
@@ -1470,7 +1715,7 @@ namespace ojph {
     assert(file_handle == 0);
     file_handle = fopen(filename, "rb");
     if (0 == file_handle)
-      OJPH_ERROR(0x0300000D1, "Unable to open file %s", filename);
+      OJPH_ERROR(0x03000161, "Unable to open file %s", filename);
     fname = filename;
 
     // read magic number
@@ -1478,7 +1723,7 @@ namespace ojph {
     if (fread(&magic_number, sizeof(ui32), 1, file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000D2, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000162, "Error reading file %s", filename);
     }
 
     // check magic number
@@ -1497,7 +1742,7 @@ namespace ojph {
     else
     {
       close();
-      OJPH_ERROR(0x0300000D3, "Error reading file %s - this does not appear "
+      OJPH_ERROR(0x03000163, "Error reading file %s - this does not appear "
         "to be a valid DPX file.  It has magic number = 0x%08X.  The magic "
         "number of a DPX file is 0x%08X.", filename, magic_number, 
         dpx_magic_number);
@@ -1508,7 +1753,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000D4, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000164, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       offset_to_image_data_in_bytes = be2le(offset_to_image_data_in_bytes);
@@ -1516,14 +1761,14 @@ namespace ojph {
     if (fread(version, sizeof(uint8_t), 8, file_handle) != 8)
     {
       close();
-      OJPH_ERROR(0x0300000D5, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000165, "Error reading file %s", filename);
     }
     // read image file size in bytes
     if (fread(&total_image_file_size_in_bytes, sizeof(ui32), 1, file_handle) 
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000D6, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000166, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       total_image_file_size_in_bytes = be2le(total_image_file_size_in_bytes);
@@ -1532,14 +1777,14 @@ namespace ojph {
     if (fseek(file_handle,768, SEEK_SET) != 0)
     {
       close();
-      OJPH_ERROR(0x0300000D7, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000167, "Error reading file %s", filename);
     }
 
     // read image_orientation
     if (fread(&image_orientation, sizeof(uint16_t), 1, file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000D8, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000168, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       image_orientation = be2le(image_orientation);
@@ -1549,7 +1794,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000D9, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000169, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       number_of_image_elements = be2le(number_of_image_elements);
@@ -1558,7 +1803,7 @@ namespace ojph {
     if (fread(&pixels_per_line, sizeof(ui32), 1, file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000DA, "Error reading file %s", filename);
+      OJPH_ERROR(0x0300016A, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       pixels_per_line = be2le(pixels_per_line);
@@ -1567,7 +1812,7 @@ namespace ojph {
     if (fread(&lines_per_image_element, sizeof(ui32), 1, file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000DB, "Error reading file %s", filename);
+      OJPH_ERROR(0x0300016B, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       lines_per_image_element = be2le(lines_per_image_element);
@@ -1576,7 +1821,7 @@ namespace ojph {
     if (fseek(file_handle, 780, SEEK_SET) != 0)
     {
       close();
-      OJPH_ERROR(0x0300000DC, "Error reading file %s", filename);
+      OJPH_ERROR(0x0300016C, "Error reading file %s", filename);
     }
 
     // read data sign for image element
@@ -1584,7 +1829,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000DE, "Error reading file %s", filename);
+      OJPH_ERROR(0x0300016E, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       data_sign_for_image_element_1 = be2le(data_sign_for_image_element_1);
@@ -1593,7 +1838,7 @@ namespace ojph {
     if (fseek(file_handle, 800, SEEK_SET) != 0)
     {
       close();
-      OJPH_ERROR(0x0300000DF, "Error reading file %s", filename);
+      OJPH_ERROR(0x0300016F, "Error reading file %s", filename);
     }
 
     // read descriptor
@@ -1601,7 +1846,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E0, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000170, "Error reading file %s", filename);
     }
 
     // read transfer characteristic
@@ -1609,7 +1854,7 @@ namespace ojph {
               1, file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E1, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000171, "Error reading file %s", filename);
     }
 
     // read colorimetric specification
@@ -1617,7 +1862,7 @@ namespace ojph {
         1, file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E2, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000172, "Error reading file %s", filename);
     }
 
     // read bit depth
@@ -1625,7 +1870,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E3, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000173, "Error reading file %s", filename);
     }
 
     // read packing
@@ -1633,7 +1878,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E4, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000174, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       packing_for_image_element_1 = be2le(packing_for_image_element_1);
@@ -1643,7 +1888,7 @@ namespace ojph {
         != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E5, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000175, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       encoding_for_image_element_1 = be2le(encoding_for_image_element_1);
@@ -1653,7 +1898,7 @@ namespace ojph {
               file_handle) != 1)
     {
       close();
-      OJPH_ERROR(0x0300000E6, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000176, "Error reading file %s", filename);
     }
     if (is_byte_swapping_necessary)
       offset_to_data_for_image_element_1 = 
@@ -1663,7 +1908,7 @@ namespace ojph {
     if (fseek(file_handle, (long)offset_to_image_data_in_bytes, SEEK_SET) != 0)
     {
       close();
-      OJPH_ERROR(0x0300000E7, "Error reading file %s", filename);
+      OJPH_ERROR(0x03000177, "Error reading file %s", filename);
     }
 
     // set ojph properties
@@ -1689,7 +1934,7 @@ namespace ojph {
     // allocate linebuffer to hold a line of image data from the file
     line_buffer = malloc(number_of_32_bit_words_per_line * sizeof(ui32) );
     if (NULL == line_buffer)
-      OJPH_ERROR(0x0300000E8, "Unable to allocate %d bytes for line_buffer[] "
+      OJPH_ERROR(0x03000178, "Unable to allocate %d bytes for line_buffer[] "
         "for file %s", 
         number_of_32_bit_words_per_line * sizeof(ui32), filename);
 
@@ -1697,7 +1942,7 @@ namespace ojph {
     line_buffer_16bit_samples = 
       (ui16*) malloc(width * num_comps * sizeof(ui16));
     if (NULL == line_buffer_16bit_samples)
-      OJPH_ERROR(0x0300000E9, "Unable to allocate %d bytes for "
+      OJPH_ERROR(0x03000179, "Unable to allocate %d bytes for "
         "line_buffer_16bit_samples[] for file %s", 
         width * num_comps * sizeof(ui16), filename);
 
@@ -1719,7 +1964,7 @@ namespace ojph {
           file_handle) != number_of_32_bit_words_per_line)
       {
         close();
-        OJPH_ERROR(0x0300000F1, "Error reading file %s", fname);
+        OJPH_ERROR(0x03000181, "Error reading file %s", fname);
       }
 
       if (true == is_byte_swapping_necessary)
@@ -1773,7 +2018,7 @@ namespace ojph {
       }
       else
       {
-        OJPH_ERROR(0x0300000F2, "file %s uses DPX image formats that are not "
+        OJPH_ERROR(0x03000182, "file %s uses DPX image formats that are not "
           "yet supported by this software\n bitdepth_for_image_element_1 = "
           "%d\n num_comps=%d\npacking_for_image_element_1=%d\n "
           "descriptor_for_image_element_1=%d", fname, 
