@@ -138,6 +138,7 @@ namespace ojph {
       COM = 0xFF64, //comment
       DFS = 0xFF72, //downsampling factor styles
       ADS = 0xFF73, //arbitrary decomposition styles
+      NLT = 0xFF76, //non-linearity point transformation
       ATK = 0xFF79, //arbitrary transformation kernels
       SOT = 0xFF90, //start of tile-part
       SOP = 0xFF91, //start of packet
@@ -165,13 +166,20 @@ namespace ojph {
     {
       friend ::ojph::param_siz;
 
+    public:
+      enum : ui16 {
+        RSIZ_NLT_FLAG  =  0x200,
+        RSIZ_HT_FLAG   = 0x4000,
+        RSIZ_EXT_FLAG  = 0x8000,
+      };
+
     public:
       param_siz()
       {
         memset(this, 0, sizeof(param_siz));
         cptr = store;
         old_Csiz = 4;
-        Rsiz = 0x4000; //for jph, bit 14 of Rsiz is 1
+        Rsiz = RSIZ_HT_FLAG;
       }
 
       ~param_siz()
@@ -273,6 +281,11 @@ namespace ojph {
       bool is_ws_kern_support_needed() { return ws_kern_support_needed; }
       bool is_dfs_support_needed() { return dfs_support_needed; }
 
+      void set_Rsiz_flag(ui16 flag)
+      { Rsiz |= flag; }
+      void reset_Rsiz_flag(ui16 flag)
+      { Rsiz = (ui16)(Rsiz & ~flag); }
+
     private:
       ui16 Lsiz;
       ui16 Rsiz;
@@ -659,6 +672,64 @@ namespace ojph {
         ui16 comp_idx;
     };
 
+    ///////////////////////////////////////////////////////////////////////////
+    //
+    //
+    //
+    //
+    //
+    ///////////////////////////////////////////////////////////////////////////
+    // data structures used by param_nlt
+    struct param_nlt
+    {
+      using special_comp_num = ojph::param_nlt::special_comp_num;
+    public:
+      param_nlt() { 
+        Lnlt = 6;
+        Cnlt = special_comp_num::ALL_COMPS; // default
+        BDnlt = 0;
+        Tnlt = 3;
+        enabled = false; next = NULL; alloced_next = false;
+      }
+
+      ~param_nlt() {
+        if (next && alloced_next) {
+          delete next;
+          alloced_next = false;
+          next = NULL;
+        }
+      }
+
+      void check_validity(param_siz& siz);
+      void set_type3_transformation(ui32 comp_num, bool enable);
+      bool get_type3_transformation(ui32 comp_num, ui8& bit_depth, 
+                                    bool& is_signed) const;
+      bool write(outfile_base* file) const;
+      void read(infile_base* file);
+
+    private:
+      const param_nlt* get_comp_object(ui32 comp_num) const;
+      param_nlt* get_comp_object(ui32 comp_num);
+      param_nlt* add_object(ui32 comp_num);
+      bool is_any_enabled() const;
+      void trim_non_existing_components(ui32 num_comps);
+
+    private:
+      ui16 Lnlt;         // length of the marker segment excluding marker
+      ui16 Cnlt;         // Component involved in the transformation
+      ui8 BDnlt;         // Decoded image component bit depth parameter
+      ui8 Tnlt;          // Type of non-linearity
+      bool enabled;      // true if this object is used
+      param_nlt* next;   // for chaining NLT markers
+      bool alloced_next; // true if next was allocated, not just set to an
+                         // existing object
+
+      // The top level param_nlt object is not allocated, but as part of 
+      // codestream, and is used to manage allocated next objects.
+      // next holds a list of param_nlt objects, which are managed by the top
+      // param_nlt object.
+    };
+
     ///////////////////////////////////////////////////////////////////////////
     //
     //
